/* ============================================================
 *
 * This file is a part of CCA project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <cmath>

#include <iostream>
#include <sstream>
#include <stdexcept>

#include <boost/shared_ptr.hpp>
#include <boost/assign/std/vector.hpp>

#include <rsc/runtime/Printable.h>

#include <nemo/Vector.h>

namespace cca {

using namespace boost::assign;

/**
 * Mapping: degree to radians
 */
class DegToRad: public nemo::MappingDefinition<double, double> {
public:
    static nemo::Mapping<double, double> create() {
        return nemo::Mapping<double, double>(new DegToRad());
    }

    double evaluate(double value) const {
        return value / 180 * M_PI;
    }
};
static nemo::Mapping<double, double> degToRad = DegToRad::create();

/**
 * Mapping: radians to degrees
 */
class RadToDeg: public nemo::MappingDefinition<double, double> {
public:
    static nemo::Mapping<double, double> create() {
        return nemo::Mapping<double, double>(new RadToDeg());
    }

    double evaluate(double value) const {
        return value / M_PI * 180;
    }
};
static nemo::Mapping<double, double> radToDeg = RadToDeg::create();

class ContainingDoubles;
typedef boost::shared_ptr<ContainingDoubles> ContainingDoublesPtr;

/**
 * Base for DTOs that contain double values or have setters/getters for double
 * values.
 *
 * @ingroup DTO
 */
class ContainingDoubles {

public:
    /**
     * Constructor with empty initialization.
     */
    ContainingDoubles() :
            _values() {
    }

    /**
     * Constructor from single value. Result is a one-dimensional DTO.
     *
     * @param value
     *          Initial value
     */
    ContainingDoubles(double value) :
            _values(value) {
    }

    /**
     * Constructor from dimension and value. Result is a DTO of given dimension
     * filled with the given value.
     *
     * @param dimension
     *          Dimension of the DTO
     * @param value
     *          Initial values for all dimensions
     */
    ContainingDoubles(unsigned int dimension, double value) :
            _values(nemo::dim(dimension), value) {
    }

    /**
     * Constructor
     * Initializes DTO with specified double values.
     */
    ContainingDoubles(const nemo::RealVector& values) :
            _values(values) {
    }

    /**
     * Copy Constructor
     */
    ContainingDoubles(const ContainingDoubles& values) :
            _values(values._values) {
    }

    /**
     * Destructor
     */
    virtual ~ContainingDoubles() {
    }

    /**
     * Get dimension / number of values
     *
     * @return Number of contained values
     */
    virtual unsigned int getDimension() const {
        return this->_values.dimension();
    }

    /**
     * Returns single value as double in the default representation of this DTO.
     * You should use this method only if you know exactly what you are doing.
     * Use the more specific getters instead.
     *
     * @param index
     *          Index of the joint value
     * @return Actual value in default representation.
     */
    virtual double asDouble(unsigned int index) const {
        if (index >= this->_values.dimension()) {
            throw std::range_error("Index out of bounds.");
        }
        return this->_values[index];
    }

    /**
     * Returns single value as double. You should use this method only if you
     * know exactly what you are doing. Use the more specific getters instead.
     *
     * @return Actual values in default representation.
     */
    virtual nemo::RealVector asDoubleVector() const {
        return this->_values;
    }

    virtual void setValues(const nemo::RealVector& values) {
        if (values.dimension() != _values.dimension()) {
            std::stringstream errMsg;
            errMsg
                    << "[ContainingDoubles::setValues] Dimension mismatch: got vector of dim="
                    << values.dimension() << " but dim=" << _values.dimension()
                    << "!";
            throw std::runtime_error(errMsg.str());
        }
        this->_values = values;
    }

    virtual void setValue(double value) {
        if (this->getDimension() != 1) {
            throw std::runtime_error(
                    "This call is only valid for one-dimensional DTO.");
        }
        this->_values[0] = value;
    }

    virtual void setValue(unsigned int index, double value) {
        if (index >= this->getDimension()) {
            throw std::range_error("Index out of bounds.");
        }
        this->_values[index] = value;
    }

    /**
     * Merging several low-dimensional DTOs into this DTO.
     *
     * @todo Type safety checks? Right now I can merge arbitrary ContaingDoubles
     *          DTOs
     *
     * @tparam DTOType
     *          Type of input DTOs (automatically inferred from inputs if left
     *          blank)
     * @param inputDTOs
     *          Vector of ContainingDouble DTOs, that should be merged
     *          (in-place)
     */
    template<class DTOType>
    void merge(const std::vector<boost::shared_ptr<DTOType> >& inputDTOs) {
        // Check dimensions of input DTOs and my own dimension
        unsigned int inputdimension = 0;
        for (unsigned int inputdto = 0; inputdto < inputDTOs.size();
                ++inputdto) {
            inputdimension += inputDTOs[inputdto]->getDimension();
        }
        if (inputdimension != this->getDimension()) {
            throw std::runtime_error(
                    "Merging failed: My own dimension and the dimension of the summarized input DTOs doesn`t agree.");
        }

        // Start merging
        unsigned int cursor = 0;
        for (unsigned int inputdto = 0; inputdto < inputDTOs.size();
                ++inputdto) {
            for (unsigned int inputdim = 0;
                    inputdim < inputDTOs[inputdto]->getDimension();
                    ++inputdim) {
                this->setValue(cursor++,
                        inputDTOs[inputdto]->asDouble(inputdim));
            }
        }
    }

    /**
     * Split this DTO into several one-dimensional DTOs.
     *
     * @todo Type safety checks? Right now I can merge arbitrary ContaingDoubles
     *          DTOs
     *
     * @tparam DTOType
     *          Type of input and return DTOs
     *
     * @return Vector of one-dimensional DTOs
     */
    template<class DTOType>
    std::vector<boost::shared_ptr<DTOType> > split() const {
        std::vector<boost::shared_ptr<DTOType> > vec;

        for (unsigned int cursor = 0; cursor < getDimension(); ++cursor) {
            vec.push_back(
                    boost::shared_ptr < DTOType
                            > (new DTOType(this->asDouble(cursor))));
        }

        return vec;
    }

    /**
     * Split this DTO into several one-dimensional DTOs.
     *
     * @todo Type safety checks? Right now I can merge arbitrary ContaingDoubles
     *          DTOs
     *
     * @tparam DTOType
     *          Type of input and return DTOs
     * @param sizes
     *          Integer array specifying the sizes chunks to split into. Sum
     *          of sizes needs to match input dimension.
     *
     * @return Vector of one-dimensional DTOs
     */
    template<class DTOType>
    std::vector<boost::shared_ptr<DTOType> > split(
            nemo::IntVector sizes) const {
        unsigned int dim = 0;
        for (unsigned int i = 0; i < sizes.dimension(); ++i) {
            dim += sizes[i];
        }

        if (dim != this->getDimension()) {
            throw std::runtime_error(
                    "split() failed: My own dimension and the requested splitting dimensions don`t agree.");
        }

        std::vector<boost::shared_ptr<DTOType> > vec;

        unsigned int cursor = 0;
        for (unsigned int dtos = 0; dtos < sizes.dimension(); ++dtos) {
            typename boost::shared_ptr<DTOType> tmp = DTOType::create(
                    sizes[dtos], 0.0);

            for (unsigned int localcursor = 0; localcursor < sizes[dtos];
                    ++localcursor) {
                tmp->setValue(localcursor, this->asDouble(cursor++));
            }

            vec.push_back(tmp);
        }

        return vec;
    }

// TODO: Clean-up - now we want that?
//protected:

    /**
     *
     */
    nemo::RealVector _values;
};

}
