/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <iostream>
#include <vector>

#include <boost/shared_ptr.hpp>

#include "ControlAspect.h"

namespace rci {

class ControlMode;
typedef boost::shared_ptr<ControlMode> ControlModePtr;

/**
 * Domain object indicating control mode of a controller / controlled device.
 * A ControlMode can itself contain several other control modes.
 * @ingroup DTO
 * @todo Should have constructor with ControlMode vector as third parameter
 * @todo Control Mode combination should be commutative.
 */
class ControlMode {

public:

    /**
     * Constructor, initializing control to desired mode.
     * @param name Name of the control mode
     * @return Control mode object, set to 0 (disabled)
     */
    ControlMode(std::string name);

    /**
     * Destructor
     */
    virtual ~ControlMode();

    /**
     * Returns name of the control mode instance.
     * @return Speking name
     */
    std::string getName() const;

    /**
     * Returns wheather this ControlMode is the passed one.
     * @param mode ControlMode to compare with.
     * @return Are the two ControlModes equal to each other?
     */
    bool equals(const ControlModePtr &mode) const;

    /**
     * Returns wheather this ControlMode is the passed one or contains it.
     * @param mode ControlMode to compare with.
     * @return Is or contains this ControlMode the passed one?
     */
    bool supports(const ControlAspectPtr &mode) const;

    /**
     * Add control mode
     */
    void addControlAspect(ControlAspectPtr mode);

    /**
     * Add control mode
     */
    ControlModePtr combineWith(const ControlModePtr &mode,
            const std::string &name) const;

    /**
     * Print method
     * @return Contrrol mode represented as string
     */
    std::string print() const;

    /**
     * Initializes control to desired mode.
     * @param name Name of the control mode
     * @return Control mode object, set to 0 (disabled)
     */
    static ControlModePtr create(std::string name);

    /**
     * Initializes control to desired mode.
     * @param name Name of the control mode
     * @param aspect Contained control aspect
     * @return Control mode object, set to 0 (disabled)
     */
    static ControlModePtr create(std::string name, ControlAspectPtr aspect);

    /**
     * Initializes control to desired mode.
     * @param name Name of the control mode
     * @param aspect1 Contained control aspect
     * @param aspect2 Contained control aspect
     * @return Control mode object, set to 0 (disabled)
     */
    static ControlModePtr create(std::string name, ControlAspectPtr aspect1,
            ControlAspectPtr aspect2);

protected:
    /**
     * Name of the control mode.
     */
    std::string name;

    /**
     * Contaiend control modes.
     * A ControlMode can optionally contain other control modes, allowing
     * hierarchical stacking of control modes.
     */
    std::vector<ControlAspectPtr> contained;

    /**
     * Returns number of control aspects.
     */
    unsigned int numAspects() const;

private:

    /**
     * Copy constructor. Private to disallow copying.
     */
    ControlMode(const ControlMode& controlmode);
};

/**
 * Definition of default ControlMode representing inactive control.
 */
static ControlModePtr modeNoControl = ControlMode::create(
        "Inactive / no Control", noControl);

/**
 * Definition of default ControlMode representing Pose Control (cartesian
 * control).
 */
static ControlModePtr modePoseControl = ControlMode::create(
        "Cartesian Pose Control", poseControl);

/**
 * Definition of default ControlMode representing Pose Control (cartesian
 * control).
 */
static ControlModePtr modeForceControl = ControlMode::create(
        "Cartesian Force Control", forceControl);

/**
 * Definition of default ControlMode representing Joint Position Control (joint
 * angles).
 */
static ControlModePtr modeJointPositionControl = ControlMode::create(
        "Joint Position Control", jointPositionControl);

/* Compliance */

/**
 * Definition of default ControlMode representing Joint Compliance Control
 * (joint stiffness and damping for the joints).
 */
static ControlModePtr modeJointComplianceControl = ControlMode::create(
        "Joint Compliance Control", jointComplianceControl);

/**
 * Definition of default ControlMode representing Cartesian Compliance Control
 * (stiffness and damping in task space).
 */
static ControlModePtr modeCartesianComplianceControl = ControlMode::create(
        "Cartesian Compliance Control", cartesianComplianceControl);

/**
 * Definition of default ControlMode representing Joint Velocity Control.
 */
static ControlModePtr modeJointVelocityControl = ControlMode::create(
        "Joint Velocity Control", jointVelocityControl);

/**
 * Definition of default ControlMode representing Joint Torque Control.
 */
static ControlModePtr modeJointTorqueControl = ControlMode::create(
        "Joint Torque Control", jointTorqueControl);
}
