/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <iostream>

#include "rci/ControlMode.h"
#include "rci/dto/JointAngles.h"
#include "rci/dto/JointTorques.h"
#include "rci/dto/JointImpedance.h"
#include "rci/dto/JointVelocities.h"
#include "rci/dto/Pressure.h"
#include "rci/dto/Length.h"
#include "rci/dto/CartesianPose.h"

namespace rci {

class Controlled {

public:
    /**
     * Controller converged
     *
     * Answers the question, if controller converged after last command.
     */
    virtual bool isConverged() const = 0;

    /**
     * Control mode getter
     *
     * @return Current control mode
     */
    virtual ControlModePtr getControlMode();

    /**
     * Control mode setter
     *
     * @param controlMode Desired control mode
     */
    virtual void setControlMode(ControlModePtr controlMode);

protected:
    ControlModePtr _controlMode;

    Controlled();
    virtual ~Controlled();
};

/**
 * Interface indicating Joint Position Control
 *
 * Interface indicating Joint Position Control on a Node.
 */
class PositionControlled {

public:
    /**
     * Sets position
     *
     * Sets a position on a joint/actuator (joint space control).
     */
    virtual bool setJointPosition(JointAnglesPtr position);

    /**
     * Returns latest position command. Note, that this is the latest valid
     * commanded position. If the position command exceeds limits and is
     * therefore rejected, it ..
     */
    virtual JointAnglesPtr getLastPositionCommand() const;

protected:
    PositionControlled();
    virtual ~PositionControlled();

    JointAnglesPtr _lastCommandedPosition;
};

/**
 * Interface indicating Joint Velocity Control
 *
 * Interface indicating Joint Velocity Control on a Node.
 */
class VelocityControlled {

public:
    /**
     * Sets position
     *
     * Sets a position on a joint/actuator (joint space control).
     */
    virtual bool setJointVelocity(JointVelocitiesPtr velocity);

    /**
     * Returns latest position command. Note, that this is the latest valid
     * commanded position. If the position command exceeds limits and is
     * therefore rejected, it ..
     */
    virtual JointVelocitiesPtr getLastVelocityCommand() const;

protected:
    VelocityControlled();
    virtual ~VelocityControlled();

    JointVelocitiesPtr _lastCommandedVelocity;
};

/**
 * Interface indicating Joint Torque Control
 *
 * Interface indicating Joint Torque Control on a Node.
 */
class TorqueControlled {

public:
    /**
     * Sets position
     *
     * Sets a position on a joint/actuator (joint space control).
     */
    virtual bool setJointTorque(JointTorquesPtr position);

    /**
     * Returns latest position command. Note, that this is the latest valid
     * commanded position. If the position command exceeds limits and is
     * therefore rejected, it ..
     */
    virtual JointTorquesPtr getLastTorqueCommand() const;

protected:
    TorqueControlled();
    virtual ~TorqueControlled();

    JointTorquesPtr _lastCommandedTorque;
};

/**
 * Interface indicating Joint Impedance Control
 *
 * Interface indicating Joint Impedance Control on a Node.
 */
class ImpedanceControlled {

public:
    /**
     * Sets position
     *
     * Sets a position on a joint/actuator (joint space control).
     */
    virtual bool setJointImpedance(JointImpedancePtr impedance);

    /**
     * Returns latest position command. Note, that this is the latest valid
     * commanded position. If the position command exceeds limits and is
     * therefore rejected, it ..
     */
    virtual JointImpedancePtr getLastImpedanceCommand() const;

protected:
    ImpedanceControlled();
    virtual ~ImpedanceControlled();

    JointImpedancePtr _lastCommandedImpedance;
};

/**
 * Interface indicating Pressure Control on an actuated node
 *
 * Interface indicating Pressure Control on an actuated node.
 */
class PressureControlled {

public:
    /**
     * Sets position
     *
     * Sets a position on a joint/actuator (joint space control).
     */
    virtual bool setPressure(PressurePtr pressure);

    /**
     * Returns latest position command. Note, that this is the latest valid
     * commanded position. If the position command exceeds limits and is
     * therefore rejected, it ..
     */
    virtual PressurePtr getLastPressureCommand() const;

protected:
    PressureControlled();
    virtual ~PressureControlled();

    PressurePtr _lastCommandedPressure;
};

/**
 * Interface indicating Pressure Control on an actuated node
 *
 * Interface indicating Pressure Control on an actuated node.
 */
class LengthControlled {

public:
    /**
     * Sets position
     *
     * Sets a position on a joint/actuator (joint space control).
     */
    virtual bool setLength(LengthPtr Length);

    /**
     * Returns latest position command. Note, that this is the latest valid
     * commanded position. If the position command exceeds limits and is
     * therefore rejected, it ..
     */
    virtual LengthPtr getLastLengthCommand() const;

protected:
    LengthControlled();
    virtual ~LengthControlled();

    LengthPtr _lastCommandedLength;
};

/**
 * Interface indicating Cartesian Pose Control
 *
 * Interface indicating Cartesian Pose Control on a Node.
 */
class PoseControlled {

public:
    /**
     * Sets a cartesian pose
     *
     * Sets a cartesian pose on a end-effector (task space control).
     */
    virtual bool setPose(PosePtr pose);

    /**
     * Returns latest pose command. Note, that this is the latest valid
     * commanded pose. If the pose command exceeds limits and is
     * therefore rejected, it ..
     */
    virtual PosePtr getLastPoseCommand() const;

protected:
    PoseControlled();
    virtual ~PoseControlled();

    PosePtr _lastCommandedPose;
};

}

