/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#include "JointAngles.h"

using namespace std;
using namespace nemo;
using namespace rsc::math;
using namespace cca;

namespace rci {

JointAngles::JointAngles() :
		JointValues() {
}

JointAngles::JointAngles(double value) :
		JointValues(value) {
}

JointAngles::JointAngles(const RealVector& values) :
		JointValues(values) {
}

JointAngles::JointAngles(const JointAngles& values) :
		JointValues(values) {
}

double JointAngles::asDouble(unsigned int index, bool norm) const {
	if (index >= this->getDimension()) {
		throw std::range_error("Index out of bounds.");
	}
	if (!norm) {
		return this->_values[index];
	} else {
		return MathUtils::normalizeAngle(this->_values[index]);
	}
}

double JointAngles::rad(unsigned int index, bool norm) const {
	return this->asDouble(index, norm);
}

nemo::RealVector JointAngles::radVector(bool norm) const {
	return this->asDoubleVector(norm);
}

double JointAngles::deg(unsigned int index, bool norm) const {
	return (this->asDouble(index, norm) / M_PI * 180);
}

nemo::RealVector JointAngles::degVector(bool norm) const {
	return this->asDoubleVector(norm).map(cca::radToDeg);
}

nemo::RealVector JointAngles::asDoubleVector(bool norm) const {
	if (!norm) {
		return this->_values;
	} else {
		RealVector vector(dim(this->getDimension()));
		// TODO: Use mapping
		for (unsigned int i = 0; i < this->getDimension(); ++i) {
			vector[i] = MathUtils::normalizeAngle(this->_values[i]);
		}
		return vector;
	}
}

JointAnglesPtr JointAngles::create(unsigned int dimension, double value) {
	RealVector values = RealVector(dim(dimension), value);
	return JointAngles::fromRad(values);
}

void JointAngles::setFromRad(unsigned int index, double value) {
	if (index >= _values.dimension()) {
		std::stringstream errMsg;
		errMsg << "[JointAngles::setFromRad] Dimension mismatch: got index="
				<< index << " but dim=" << _values.dimension() << "!";
		throw std::runtime_error(errMsg.str());
	}

	_values[index] = value;
}

void JointAngles::setFromRad(const RealVector& values) {
	if (values.dimension() != _values.dimension()) {
		std::stringstream errMsg;
		errMsg
				<< "[JointAngles::setFromRad] Dimension mismatch: got vector of dim="
				<< values.dimension() << " but dim=" << _values.dimension()
				<< "!";
		throw std::runtime_error(errMsg.str());
	}

	_values = values;
}

void JointAngles::setFromDeg(unsigned int index, double value) {
	if (index >= _values.dimension()) {
		std::stringstream errMsg;
		errMsg << "[JointAngles::setFromDeg] Dimension mismatch: got index="
				<< index << " but dim=" << _values.dimension() << "!";
		throw std::runtime_error(errMsg.str());
	}

	_values[index] = cca::degToRad(value);
}

void JointAngles::setFromDeg(const RealVector& values) {
	if (values.dimension() != _values.dimension()) {
		std::stringstream errMsg;
		errMsg
				<< "[JointAngles::setFromDeg] Dimension mismatch: got vector of dim="
				<< values.dimension() << " but dim=" << _values.dimension()
				<< "!";
		throw std::runtime_error(errMsg.str());
	}

	_values = values.map(cca::degToRad);
}

JointAnglesPtr JointAngles::copy(const JointAngles& angles) {
	return JointAnglesPtr(new JointAngles(angles));
}

JointAnglesPtr JointAngles::fromRad(const nemo::RealVector& values) {
	return JointAnglesPtr(new JointAngles(values));
}

JointAnglesPtr JointAngles::fromRad(double value) {
	return JointAngles::fromValue(value);
}

JointAnglesPtr JointAngles::fromDeg(const nemo::RealVector& values) {
	return JointAngles::fromRad(values.map(cca::degToRad));
}

JointAnglesPtr JointAngles::fromDeg(double value) {
	return JointAngles::fromValue(value / 180 * M_PI);
}

void JointAngles::normalize() {
	// There probably is a more efficient way using std vector methods
	for (unsigned int i = 0; i < this->getDimension(); ++i) {
		this->_values[i] = MathUtils::normalizeAngle(this->_values[i]);
	}
}

std::string JointAngles::print() const {
	ostringstream outstream(ostringstream::out);
	outstream.precision(3); // Precision when printing double values
	outstream << "JointAngles<";
	for (unsigned int i = 0; i < this->getDimension(); ++i) {
		if (i > 0) {
			outstream << ",";
		}
		outstream << this->asDouble(i);
	}
	outstream << ">(rad)";
	return outstream.str();
}

JointAnglesPtr JointAngles::fromValue(double value) {
	nemo::RealVector v = RealVector(value);
	JointAnglesPtr angle(new JointAngles(v));
	return angle;
}

}
