/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#include "JointImpedance.h"

using namespace std;
using namespace nemo;

namespace rci {

JointImpedance::JointImpedance() :
        JointValues(), _stiffness(), _damping() {
}

JointImpedance::JointImpedance(double stiffness, double damping) :
        JointValues(), _stiffness(stiffness), _damping(damping) {

}

JointImpedance::JointImpedance(const RealVector& values) :
        JointValues(), _stiffness(values.dimension() / 2, 0.0), _damping(
                values.dimension() / 2, 0.0) {
    for (unsigned int index = 0; index < values.dimension() / 2; ++index) {
        _stiffness.setValue(index, values[2 * index]);
        _damping.setValue(index, values[2 * index + 1]);
    }
}

JointImpedance::JointImpedance(const RealVector& svalues,
        const RealVector& dvalues) :
        JointValues(), _stiffness(svalues), _damping(dvalues) {

}

JointImpedancePtr JointImpedance::copy(const JointImpedance& imp) {
    return JointImpedancePtr(new JointImpedance(imp));
}

JointImpedancePtr JointImpedance::create(const RealVector& values) {
    return JointImpedancePtr(new JointImpedance(values));
}

unsigned int JointImpedance::getDimension() const {
    return this->_stiffness.getDimension();
}

JointImpedance::JointImpedance(JointStiffness stiffness, JointDamping damping) :
        _stiffness(stiffness), _damping(damping) {
}

JointStiffness JointImpedance::getStiffness() const {
    return this->_stiffness;
}

JointStiffness JointImpedance::getStiffness(unsigned int index) const {
    return this->_stiffness[index];
}

JointDamping JointImpedance::getDamping() const {
    return this->_damping;
}

JointDamping JointImpedance::getDamping(unsigned int index) const {
    return this->_damping[index];
}

void JointImpedance::setValue(unsigned int index, double value) {
    if (index % 2 == 0) {
        this->_stiffness.setValue(index / 2, value);
    } else {
        this->_damping.setValue(index / 2, value);
    }
}

double JointImpedance::asDouble(unsigned int index) const {
    if (index % 2 == 0) {
        return this->_stiffness.asDouble(index / 2);
    } else {
        return this->_damping.asDouble(index / 2);
    }
}

nemo::RealVector JointImpedance::asDoubleVector() const {
    RealVector values(dim(getDimension() * 2));
    for (unsigned int index = 0; index < getDimension(); ++index) {
        values[index] = this->_stiffness.asDouble(index);
        values[index + getDimension()] = this->_damping.asDouble(index);
    }
    return values;
}

JointImpedance JointImpedance::operator[](unsigned int index) const {
    return JointImpedance(this->getStiffness(index), this->getDamping(index));
}

std::string JointImpedance::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "JointImpedance< " << this->_stiffness << " , "
            << this->_damping;
    outstream << " >";
    return outstream.str();
}

JointStiffness::JointStiffness() :
        JointValues() {
}

JointStiffness::JointStiffness(double stiffness) :
        JointValues(stiffness) {
}

JointStiffness::JointStiffness(unsigned int dim, double stiffness) :
        JointValues(dim, stiffness) {
}

JointStiffness::JointStiffness(const RealVector& values) :
        JointValues(values) {
}

JointStiffness JointStiffness::operator[](unsigned int index) const {
    return JointStiffness(this->asDouble(index));
}

std::string JointStiffness::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "JointStiffness<" << this->_values << ">";
    return outstream.str();
}

JointDamping::JointDamping() :
        JointValues() {
}

JointDamping::JointDamping(double damping) :
        JointValues(damping) {
}

JointDamping::JointDamping(unsigned int dim, double damping) :
        JointValues(dim, damping) {
}

JointDamping::JointDamping(const RealVector& values) :
        JointValues(values) {
}

JointDamping JointDamping::operator[](unsigned int index) const {
    return JointDamping(this->asDouble(index));
}

std::string JointDamping::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "JointDamping<" << this->_values << ">";
    return outstream.str();
}

}
