/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#include "JointTorques.h"

using namespace std;
using namespace nemo;

namespace rci {

JointTorquesPtr JointTorques::fromNm(double value) {
    return JointTorquesPtr(new JointTorques(RealVector(value)));
}

JointTorquesPtr JointTorques::fromNm(const nemo::RealVector& values) {
    return JointTorquesPtr(new JointTorques(values));
}

JointTorques::JointTorques() :
        JointValues() {
}

JointTorques::JointTorques(double value) :
        JointValues(value) {
}

JointTorques::JointTorques(const RealVector& values) :
        JointValues(values) {
}

JointTorquesPtr JointTorques::create(unsigned int dimension, double value) {
    RealVector values = RealVector(dim(dimension), value);
    return JointTorques::fromNm(values);
}

double JointTorques::Nm(unsigned int index) const {
    if (index >= _values.dimension()) {
        std::stringstream errMsg;
        errMsg << "[JointTorques::Nm] index=" << index << " out of bounds 0.."
                << _values.dimension() << "!";
        throw std::runtime_error(errMsg.str());
    }
    return _values[index];
}

nemo::RealVector JointTorques::NmVector() const {
    return _values;
}

std::string JointTorques::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "JointTorques<";
    for (unsigned int i = 0; i < this->getDimension(); ++i) {
        if (i > 0) {
            outstream << ",";
        }
        outstream << this->asDouble(i);
    }
    outstream << ">(N*cm)" << std::endl;
    return outstream.str();
}

void JointTorques::setFromNm(unsigned int index, double value) {
    if (index >= _values.dimension()) {
        std::stringstream errMsg;
        errMsg << "[JointTorques::setFromNm] index=" << index
                << " out of bounds 0.." << _values.dimension() << "!";
        throw std::runtime_error(errMsg.str());
    }
    _values[index] = value;
}

void JointTorques::setFromNm(const nemo::RealVector& values) {
    if (values.dimension() != _values.dimension()) {
        std::stringstream errMsg;
        errMsg << "[JointTorques::setFromNm] Dimension mismatch! Expected dim="
                << _values.dimension() << "  but got dim=" << values.dimension()
                << "!";
        throw std::runtime_error(errMsg.str());
    }
    _values = values;
}

JointTorquesPtr JointTorques::copy(const JointTorques& torques) {
    return JointTorquesPtr(new JointTorques(torques));
}

}
